#!/usr/bin/env node

// Usage:
// issue-report init
// issue-report [issue-file] - if no issue-file is given, it is assumed
//    to be issues.yaml in the current working directory.

const fs = require('fs-extra');
const glob = require('glob');
const htmltidy = require('htmltidy');
const path = require('path');
const R = require('ramda');
const util = require('util');
const which = require('which');
const yaml = require('js-yaml');

const mu = require('./misc-utils.js');
const IssueDb = require('./issue-db.js');
const xhtml = require('./xhtml.js');


const resolve = path.resolve;  // shortcut
const isExecutable = pathname => {
  try {
    fs.accessSync(pathname, fs.X_OK);
    return true;
  }
  catch (err) {
    return false;
  }
}

const copyFile = R.curry(
  (destDir, modify, srcPath) => {
    const contents = modify(fs.readFileSync(srcPath, 'utf8'));
    const filename = path.basename(srcPath);
    const destPath = path.resolve(destDir, filename);
    fs.writeFileSync(destPath, contents, 'utf8');
    if (isExecutable(srcPath)) {
      fs.chmodSync(destPath, 0755);
    }
    return null;
  }
);

const today = (new Date).toISOString().substr(0,10);

const dump = (label, obj) =>
  console.log(label + ': ' + util.inspect(obj, false, 7, true));


const initProject = function(projDir) {
  console.log(`Initializing a new project in ${projDir}`);

  const siteFiles = glob.sync(resolve(__dirname, 'site-files/*'));
  const nodePath = which.sync('node');
  const modify = content => content.replace(/{{node_exe}}/, nodePath);
  siteFiles.forEach(copyFile(projDir, modify));
}



// Asyncronous function that generates a report, and delivers it as an HTML
// string.

const genReport = function(opts, cb) {
  const issuesPath = (opts && opts.issuesFile) || resolve('./issues.yaml');
  const projDir = resolve(issuesPath, '..');
  const issueDb = new IssueDb(issuesPath);
  //dump('issueDb', issueDb);

  const _title = `${issueDb.title} - ${today}`;
  const jiraBase = 'https://jira.ncbi.nlm.nih.gov/browse/';

  const {a, body, h1, head, html, table, td, th, title, tr} = xhtml;

  const groupToHtml = group => {
    //dump('in groupToHtml; group = ', group);
    returnVal =
      tr(
        th(group.name, {colspan: 2}) +
        th('original', {'class': 'esthead'}) +
        th('current', {'class': 'esthead'}),
        {'class': 'group'}
      ) +
      Object.keys(group.issues).map(k =>
        issueToHtml(k, group.issues[k])).join('') +
      tr(td(' '), {'class': 'spacer'});
    return returnVal;
  };

  const issueToHtml = (k, issue) => {
    return tr(
      td(a(k, {href: jiraBase + k})) +
      td(issue.title) +
      td(issue.origEstimate(), {'class': 'est'}) +
      td(issue.currEstimate(), {'class': 'est'}),
      {'class': issue.status === 'resolved' ? 'resolved' : ''}
    );
  };

  const totalsRow = issueDb => {
    const totalOrig = mu.sum(issueDb.groups.map(g => g.totalOrigEst()));
    const totalCurr = mu.sum(issueDb.groups.map(g => g.totalCurrEst()));

    return tr(
      td('TOTALS:', {colspan: 2}) +
      td(totalOrig, {'class': 'est'}) +
      td(totalCurr, {'class': 'est'}),
      {'class': 'totals'}
    );
  };


  const reportHtml =
    html(
      head(
        title(_title) +
        `  <link rel="stylesheet" href="style.css" />`
      ) +
      body(
        h1(_title) +
        table(
          issueDb.groups.map(groupToHtml).join('') +
          totalsRow(issueDb)
        )
      )
    );

  htmltidy.tidy( reportHtml, { indent: true }, cb );
}


module.exports = {
  initProject: initProject,
  genReport: genReport,
}


/////////////////////////////////////////////////////////////////////////////
if (require.main === module) {

  // We've been invoked from the command line
  // Figure out what we're supposed to do, and the context
  try {
    const argv = process.argv;
    const numArgs = argv.length - 2;
    const arg1 = numArgs === 0 ? null : argv[2];
    const doInit = (arg1 === 'init');

    if (doInit) {
      return initProject(resolve());
    }
    else {
      const issuesFile = arg1 ? resolve(arg1) : null;

      return genReport(
        { issuesFile: issuesFile },
        function(err, report) {
          if (err) {
            console.error('genReport failed: ', err);
          }
          else {
            console.log(report);
          }
        }
      );
    }
  }
  catch(err) {
    console.error(err);
  }

}
